//////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2007-2011 Infinite Z, Inc.  All Rights Reserved.
//
//  File:       ZSpaceInterface.h
//  Content:    Sample code demonstrating use of 
//				      ZSStereo's Head Tracking in Default (simple) mode.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __ZSPACE_ZSPACE_INTERFACE_H__
#define __ZSPACE_ZSPACE_INTERFACE_H__

#ifdef _WIN32
#include <windows.h>
#endif

#include "ZSpace/Stereo/Gl/StereoGl.h"
#include "ZSpace/Tracker/TrackerSystem.h"
#include "ZSpace\common\Math\Matrix4.h"

class ZSpaceInterface
{
//////////////////////////////////////////
// EYE INITIALIZATION
// Needs to be called once per app,
//  usually on the last created window
// Only works on Windows so far
public:
  static bool InitializeEyes(
#ifdef _WIN32
    HWND hWnd
#endif
    );
private:
  static bool s_eyesInitialized;
// EYE INITIALIZATION
//////////////////////////////////////////

public:
  ZSpaceInterface();
  ~ZSpaceInterface();

  // Initialize the window.
  bool initialize(int windowX, int windowY, int windowWidth, int windowHeight);
  void clean();

  // Returns a gluLookAt compatible position, direction and up vector
  //  such that the object space origin (0,0,0) will show up in the
  //  center of the viewport on the display plane.
  // returns false if it couldn't get a default position
  bool getDefaultCameraPosition(float* pos3look3up3);

//////////////////////////////////////////
// STEREO RENDERING
//
// for each frame
// 1. call setupStereoEye for one eye
// 2. call your own render function
// 3. call clearStereoEye
// 4-6. repeat 1-3 for the second eye.
// 7. call endStereo()
public:
  // need to call these in pairs, a la
  // pushMatrix() and popMatrix()
  void setupStereoEye(zspace::stereo::StereoFrustum::StereoEye eye);
  void clearStereoEye();
  void endStereo();
private:
  bool m_eyeMatricesPushed;

  void pushStereoMatrices(zspace::stereo::StereoFrustum::StereoEye eye);
  void popStereoMatrices();
  void selectRenderContext(zspace::stereo::StereoFrustum::StereoEye eye);

private:
  // Stereo classes.
  zspace::stereo::StereoViewport* m_stereoViewport;
  zspace::stereo::StereoWindow*   m_stereoWindow;

  // Initialize the stereo window and viewport.
  void initializeStereo(int windowX, int windowY, int windowWidth, int windowHeight);

public:
  // used to manually swap eyes, if left/right fall out of sync
  void onSwapRenderTarget();

// STEREO RENDERING
//////////////////////////////////////////

//////////////////////////////////////////
// tracking stuff
public:
  void captureTargets();

  void toggleHeadTracking();

  // returns poses in ZSpace coords
  zspace::common::Matrix4 getHeadPose();

private:
  zspace::tracker::TrackerSystem*   m_trackerSystem;
  bool m_isHeadTrackingEnabled;
// tracking stuff
//////////////////////////////////////////


/////////////////////////////////////////
// Window management, callbacks
public:
  void onMove(int x, int y);
  void onResize(int width, int height);

  int getWindowX()      const;
  int getWindowY()      const;
  int getWindowWidth()  const;
  int getWindowHeight() const;

};

#endif // __ZSPACE_ZSPACE_INTERFACE_H__