//------------------------------------------------------------------------------
//
// npiz.cpp
//
// ANTZ - 3D data viewer with realtime real-world interaction based on NPE.
//
// ANTZ is hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
// Written in 2010-2012 by Shane Saxon - contact@saxondigital.net
//
// Additional code contributed by: Mark Buchanan and Lucas Erickson.
//
// To the extent possible under law, the author(s) have dedicated all copyright 
// and related and neighboring rights to this software to the public domain
// worldwide. This software is distributed without any warranty.
//
// Released under the CC0 license, which is GPL compatible.
//
// You should have received a copy of the CC0 Public Domain Dedication along
// with this software (license file named COPYING.txt). If not, see
// http://creativecommons.org/publicdomain/zero/1.0/
//
//------------------------------------------------------------------------------

#include "npiz.h"


//////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2007-2011 Infinite Z, Inc.  All Rights Reserved.
//
//  File:       HeadTrackingGlutWindow.cpp
//  Content:    Sample code demonstrating use of 
//				      ZSStereo's Head Tracking in Default (simple) mode.
//
//////////////////////////////////////////////////////////////////////////////

#include "HeadTrackingGlutWindow.h"

  // Define the screen resolution for windowed mode
#define WINDOWED_WIDTH	1920		//S3D was 1024x768
#define WINDOWED_HEIGHT	1080

Scene* scene;// = new Scene();				//zziz

//-----------------------------------------------------------------------------
void npInitIZ (void* dataRef)
{
  // Program entry point, marlon IZ
  // set up logging
  // In the main function of your application create the Log singleton to log
  // all messages of severity INFO and above (this includes ERR and WARN).
  zspace::common::Log::create( "HeadTracking.log", zspace::common::Log::INFO );
  ZS_LOG(INFO, "HeadTracking startup.\n" ); 


//  glutInit (&argc, argv);	//S3D

  // GLUT Window Initialization:
  // NOTE: windows are singletons because glut callbacks do not
  //  specify which window it is attached to.  This was one way
  //  to map global/static functions to an object.
  HeadTrackingGlutWindow* glutWindow = HeadTrackingGlutWindow::GetInstance();


  // GLUT window creation and global OpenGL setup wrapped in here

//S3D


  if(!glutWindow->initialize(0, 0, WINDOWED_WIDTH, WINDOWED_HEIGHT))
  {
    glutWindow->Clean();
    ZS_LOG(INFO, "Error initializing HeadTracking window.\n"); 
    zspace::common::Log::close();
    exit(-1);	//S3D
  }

  // initializeEyes is called after the window has been created
  //  with initialize(), but before the ZSpace objects are created
  //  in initializeStereo()
  if(!glutWindow->initializeEyes())
  {
    glutWindow->Clean();
    ZS_LOG(INFO, "Error synchronizing eyes.\n"); 
    zspace::common::Log::close();
    exit(-1);	//S3D
  }
  if(!glutWindow->initializeStereo(0, 0, WINDOWED_WIDTH, WINDOWED_HEIGHT))
  {
    glutWindow->Clean();
    ZS_LOG(INFO, "Error initializing stereo window.\n"); 
    zspace::common::Log::close();
    exit(-1);	//S3D
  }

  // the scene is wholly separate from the windowing system,
  //  and whether or not it is in stereo
  scene = new Scene();
  scene->initialize();

  float camera[9];
  glutWindow->getDefaultCameraPosition(camera);				//S3D update with antz cam
  scene->lookAt(camera[0], camera[1], camera[2],
                camera[3], camera[4], camera[5],
                camera[6], camera[7], camera[8]);

  glutWindow->setScene(scene);
}

//-----------------------------------------------------------------------------
void npCloseIZ (void* dataRef)
{
  // Clean up OpenGL and ZSpace objects
  // HACK: this also deletes the singleton instance
  //  since writing a singleton destroyer would
  //  probably overcomplicate a demo
  HeadTrackingGlutWindow::Clean();
  delete scene;

  ZS_LOG(INFO, "HeadTracking shutdown.\n"); 
  zspace::common::Log::close();
}


